/* See LICENSE for licence details. */
enum {
	/*
		not defined in: platforms/android-9/arch-arm/usr/include/android/keycodes.h
		copy from     : platforms/android-17/arch-arm/usr/include/android/keycodes.h
	*/
	AKEYCODE_ESCAPE      = 111,
	AKEYCODE_FORWARD_DEL = 112,
	AKEYCODE_CTRL_LEFT   = 113,
	AKEYCODE_CTRL_RIGHT  = 114,
	AKEYCODE_CAPS_LOCK   = 115,
	AKEYCODE_MOVE_HOME   = 122,
	AKEYCODE_MOVE_END    = 123,
	AKEYCODE_INSERT      = 124,
	AKEYCODE_F1          = 131,
	AKEYCODE_F2          = 132,
	AKEYCODE_F3          = 133,
	AKEYCODE_F4          = 134,
	AKEYCODE_F5          = 135,
	AKEYCODE_F6          = 136,
	AKEYCODE_F7          = 137,
	AKEYCODE_F8          = 138,
	AKEYCODE_F9          = 139,
	AKEYCODE_F10         = 140,
	AKEYCODE_F11         = 141,
	AKEYCODE_F12         = 142,
	AKEYCODE_APP_SWITCH  = 187,
};

const uint8_t keycode2keysym_table[] = {
	/* us qwerty keybord */
	/* row 4 */
	[AKEYCODE_GRAVE] = 0x60,
	[AKEYCODE_1] = 0x31, [AKEYCODE_2] = 0x32, [AKEYCODE_3] = 0x33, [AKEYCODE_4] = 0x34, [AKEYCODE_5] = 0x35,
	[AKEYCODE_6] = 0x36, [AKEYCODE_7] = 0x37, [AKEYCODE_8] = 0x38, [AKEYCODE_9] = 0x39, [AKEYCODE_0] = 0x30,
	[AKEYCODE_MINUS] = 0x2D, [AKEYCODE_EQUALS] = 0x3D,
	/* row 3 */
	[AKEYCODE_Q] = 0x71, [AKEYCODE_W] = 0x77, [AKEYCODE_E] = 0x65, [AKEYCODE_R] = 0x72, [AKEYCODE_T] = 0x74,
	[AKEYCODE_Y] = 0x79, [AKEYCODE_U] = 0x75, [AKEYCODE_I] = 0x69, [AKEYCODE_O] = 0x6F, [AKEYCODE_P] = 0x70,
	[AKEYCODE_LEFT_BRACKET] = 0x5B, [AKEYCODE_RIGHT_BRACKET] = 0x5D, [AKEYCODE_BACKSLASH] = 0x5C,
	/* row 2 */
	[AKEYCODE_A] = 0x61, [AKEYCODE_S] = 0x73, [AKEYCODE_D] = 0x64, [AKEYCODE_F] = 0x66, [AKEYCODE_G] = 0x67,
	[AKEYCODE_H] = 0x68, [AKEYCODE_J] = 0x6A, [AKEYCODE_K] = 0x6B, [AKEYCODE_L] = 0x6C,
	[AKEYCODE_SEMICOLON] = 0x3B, [AKEYCODE_APOSTROPHE] = 0x27,
	/* row 1 */
	[AKEYCODE_Z] = 0x7A, [AKEYCODE_X] = 0x78, [AKEYCODE_C] = 0x63, [AKEYCODE_V] = 0x76, [AKEYCODE_B] = 0x62,
	[AKEYCODE_N] = 0x6E, [AKEYCODE_M] = 0x6D,
	[AKEYCODE_PERIOD] = 0x2E, [AKEYCODE_COMMA] = 0x2C, [AKEYCODE_SLASH] = 0x2F,
	/* special keys */
	[AKEYCODE_TAB]    = 0x09,
	[AKEYCODE_SPACE]  = 0x20,
	[AKEYCODE_ENTER]  = 0x0A,
	[AKEYCODE_ESCAPE] = 0x1B,
	[AKEYCODE_DEL]    = 0x7F, /* backspace key */
};

const uint8_t keycode_shift[] = {
	/* row 4 */
	['`'] = '~',
	['1'] = '!', ['2'] = '@', ['3'] = '#', ['4'] = '$', ['5'] = '%',
	['6'] = '^', ['7'] = '&', ['8'] = '*', ['9'] = '(', ['0'] = ')', 
	['-'] = '_', ['='] = '+',
	/* row 3 */
	['['] = '{', [']'] = '}', ['\\'] = '|',
	/* row 2 */
	[';'] = ':', [0x27] = '"',
	/* row 1 */
	['.'] = '>', [','] = '<', ['/'] = '?',
};

const uint8_t keycode_ctrl[] = {
	['@'] = 0x00, /* FIXME: is this correct? */
	['2'] = 0x00, /* FIXME: is this correct? */
	['a'] = 0x01, ['b'] = 0x02, ['c'] = 0x03, ['d'] = 0x04, ['e'] = 0x05,
	['f'] = 0x06, ['g'] = 0x07, ['h'] = 0x08, ['i'] = 0x09, ['j'] = 0x0A,
	['k'] = 0x0B, ['l'] = 0x0C, ['m'] = 0x0D, ['n'] = 0x0E, ['o'] = 0x0F,
	['p'] = 0x10, ['q'] = 0x11, ['r'] = 0x12, ['s'] = 0x13, ['t'] = 0x14,
	['u'] = 0x15, ['v'] = 0x16, ['w'] = 0x17, ['x'] = 0x18, ['y'] = 0x19,
	['z'] = 0x1A,
	['['] = 0x1B, ['\\'] = 0x1C, [']'] = 0x1D, ['^'] = 0x1E, ['_'] = 0x1E,
};

const char *keycode2keyseq[] = {
/*
kcub1=\E[D, kcud1=\E[B, kcuf1=\E[C, kcuu1=\E[A,
*/
	[AKEYCODE_DPAD_UP]    = "\033[A",
	[AKEYCODE_DPAD_DOWN]  = "\033[B",
	[AKEYCODE_DPAD_RIGHT] = "\033[C",
	[AKEYCODE_DPAD_LEFT]  = "\033[D",
/*
kb2=\E[G, kbs=\177, kcbt=\E[Z,
*/
	/* where is backtab key? */
	[AKEYCODE_DPAD_CENTER] = "\033[G",
/*
khome=\E[1~, kich1=\E[2~, kdch1=\E[3~, kend=\E[4~,
*/
	[AKEYCODE_MOVE_HOME]   = "\033[1~",
	[AKEYCODE_INSERT]      = "\033[2~",
	[AKEYCODE_FORWARD_DEL] = "\033[3~", /* delete key */
	[AKEYCODE_MOVE_END]    = "\033[4~",
/*
kmous=\E[M, knp=\E[6~, kpp=\E[5~, kspd=^Z,
*/
	/* maybe, mouse event/suspend key are not defined */
	[AKEYCODE_PAGE_UP]   = "\033[5~",
	[AKEYCODE_PAGE_DOWN] = "\033[6~",
/*
kf1=\E[[A, kf2=\E[[B, kf3=\E[[C, kf4=\E[[D, kf5=\E[[E,
kf6=\E[17~, kf7=\E[18~, kf8=\E[19~, kf9=\E[20~, kf10=\E[21~,
kf11=\E[23~, kf12=\E[24~, kf13=\E[25~, kf14=\E[26~, kf15=\E[28~,
kf16=\E[29~, kf17=\E[31~, kf18=\E[32~, kf19=\E[33~, kf20=\E[34~,
*/
	/* maybe F13 - F20 are not defined */
	[AKEYCODE_F1]  = "\033[[A",
	[AKEYCODE_F2]  = "\033[[B",
	[AKEYCODE_F3]  = "\033[[C",
	[AKEYCODE_F4]  = "\033[[D",
	[AKEYCODE_F5]  = "\033[[E",
	[AKEYCODE_F6]  = "\033[17~",
	[AKEYCODE_F7]  = "\033[18~",
	[AKEYCODE_F8]  = "\033[19~",
	[AKEYCODE_F9]  = "\033[20~",
	[AKEYCODE_F10] = "\033[21~",
	[AKEYCODE_F11] = "\033[23~",
	[AKEYCODE_F12] = "\033[24~",
};

/* other keys
[AKEYCODE_UNKNOWN]
[AKEYCODE_SOFT_LEFT]
[AKEYCODE_SOFT_RIGHT]
[AKEYCODE_HOME]
[AKEYCODE_BACK]
[AKEYCODE_CALL]
[AKEYCODE_ENDCALL]

[AKEYCODE_STAR]
[AKEYCODE_POUND]

[AKEYCODE_VOLUME_UP]
[AKEYCODE_VOLUME_DOWN]
[AKEYCODE_POWER]
[AKEYCODE_CAMERA]
[AKEYCODE_CLEAR]

[AKEYCODE_ALT_LEFT]
[AKEYCODE_ALT_RIGHT]
[AKEYCODE_SHIFT_LEFT]
[AKEYCODE_SHIFT_RIGHT]

[AKEYCODE_SYM]
[AKEYCODE_EXPLORER]
[AKEYCODE_ENVELOPE]

[AKEYCODE_AT]
[AKEYCODE_NUM]
[AKEYCODE_HEADSETHOOK]
[AKEYCODE_FOCUS]
[AKEYCODE_PLUS]
[AKEYCODE_MENU]
[AKEYCODE_NOTIFICATION]
[AKEYCODE_SEARCH]
[AKEYCODE_MEDIA_PLAY_PAUSE]
[AKEYCODE_MEDIA_STOP]
[AKEYCODE_MEDIA_NEXT]
[AKEYCODE_MEDIA_PREVIOUS]
[AKEYCODE_MEDIA_REWIND]
[AKEYCODE_MEDIA_FAST_FORWARD]
[AKEYCODE_MUTE]

[AKEYCODE_PICTSYMBOLS]
[AKEYCODE_SWITCH_CHARSET]
[AKEYCODE_BUTTON_A]
[AKEYCODE_BUTTON_B]
[AKEYCODE_BUTTON_C]
[AKEYCODE_BUTTON_X]
[AKEYCODE_BUTTON_Y]
[AKEYCODE_BUTTON_Z]
[AKEYCODE_BUTTON_L1]
[AKEYCODE_BUTTON_R1]
[AKEYCODE_BUTTON_L2]
[AKEYCODE_BUTTON_R2]
[AKEYCODE_BUTTON_THUMBL]
[AKEYCODE_BUTTON_THUMBR]
[AKEYCODE_BUTTON_START]
[AKEYCODE_BUTTON_SELECT]
[AKEYCODE_BUTTON_MODE]

[AKEYCODE_CTRL_LEFT]
[AKEYCODE_CTRL_RIGHT]
[AKEYCODE_CAPS_LOCK]
[AKEYCODE_SCROLL_LOCK]
[AKEYCODE_META_LEFT]
[AKEYCODE_META_RIGHT]
[AKEYCODE_FUNCTION]
[AKEYCODE_SYSRQ]
[AKEYCODE_BREAK]

[AKEYCODE_FORWARD]
[AKEYCODE_MEDIA_PLAY]
[AKEYCODE_MEDIA_PAUSE]
[AKEYCODE_MEDIA_CLOSE]
[AKEYCODE_MEDIA_EJECT]
[AKEYCODE_MEDIA_RECORD]
[AKEYCODE_NUM_LOCK]
[AKEYCODE_NUMPAD_0]
[AKEYCODE_NUMPAD_1]
[AKEYCODE_NUMPAD_2]
[AKEYCODE_NUMPAD_3]
[AKEYCODE_NUMPAD_4]
[AKEYCODE_NUMPAD_5]
[AKEYCODE_NUMPAD_6]
[AKEYCODE_NUMPAD_7]
[AKEYCODE_NUMPAD_8]
[AKEYCODE_NUMPAD_9]
[AKEYCODE_NUMPAD_DIVIDE]
[AKEYCODE_NUMPAD_MULTIPLY]
[AKEYCODE_NUMPAD_SUBTRACT]
[AKEYCODE_NUMPAD_ADD]
[AKEYCODE_NUMPAD_DOT]
[AKEYCODE_NUMPAD_COMMA]
[AKEYCODE_NUMPAD_ENTER]
[AKEYCODE_NUMPAD_EQUALS]
[AKEYCODE_NUMPAD_LEFT_PAREN]
[AKEYCODE_NUMPAD_RIGHT_PAREN]
[AKEYCODE_VOLUME_MUTE]
[AKEYCODE_INFO]
[AKEYCODE_CHANNEL_UP]
[AKEYCODE_CHANNEL_DOWN]
[AKEYCODE_ZOOM_IN]
[AKEYCODE_ZOOM_OUT]
[AKEYCODE_TV]
[AKEYCODE_WINDOW]
[AKEYCODE_GUIDE]
[AKEYCODE_DVR]
[AKEYCODE_BOOKMARK]
[AKEYCODE_CAPTIONS]
[AKEYCODE_SETTINGS]
[AKEYCODE_TV_POWER]
[AKEYCODE_TV_INPUT]
[AKEYCODE_STB_POWER]
[AKEYCODE_STB_INPUT]
[AKEYCODE_AVR_POWER]
[AKEYCODE_AVR_INPUT]
[AKEYCODE_PROG_RED]
[AKEYCODE_PROG_GREEN]
[AKEYCODE_PROG_YELLOW]
[AKEYCODE_PROG_BLUE]
[AKEYCODE_APP_SWITCH]
[AKEYCODE_BUTTON_1]
[AKEYCODE_BUTTON_2]
[AKEYCODE_BUTTON_3]
[AKEYCODE_BUTTON_4]
[AKEYCODE_BUTTON_5]
[AKEYCODE_BUTTON_6]
[AKEYCODE_BUTTON_7]
[AKEYCODE_BUTTON_8]
[AKEYCODE_BUTTON_9]
[AKEYCODE_BUTTON_10]
[AKEYCODE_BUTTON_11]
[AKEYCODE_BUTTON_12]
[AKEYCODE_BUTTON_13]
[AKEYCODE_BUTTON_14]
[AKEYCODE_BUTTON_15]
[AKEYCODE_BUTTON_16]
[AKEYCODE_LANGUAGE_SWITCH]
[AKEYCODE_MANNER_MODE]
[AKEYCODE_3D_MODE]
[AKEYCODE_CONTACTS]
[AKEYCODE_CALENDAR]
[AKEYCODE_MUSIC]
[AKEYCODE_CALCULATOR]
[AKEYCODE_ZENKAKU_HANKAKU]
[AKEYCODE_EISU]
[AKEYCODE_MUHENKAN]
[AKEYCODE_HENKAN]
[AKEYCODE_KATAKANA_HIRAGANA]
[AKEYCODE_YEN]
[AKEYCODE_RO]
[AKEYCODE_KANA]
[AKEYCODE_ASSIST]
[AKEYCODE_BRIGHTNESS_DOWN]
[AKEYCODE_BRIGHTNESS_UP]
[AKEYCODE_MEDIA_AUDIO_TRACK]
*/
